#include "stm32f10x_conf.h"
#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "usart.h"
#include "segments.h"
#include "i2c.h"
#include "i2clcd.h"
#include "main.h"


#define EEP_PAGE_SZ	16
#define MAGIC		0xA5A5

#define UEBERSETZUNG	8192*7


GPIO_InitTypeDef GPIO_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;
I2C_InitTypeDef  I2C_InitStructure;
USART_InitTypeDef USART_InitStructure;
ErrorStatus HSEStartUpStatus;

// forward declarations

void RCC_Configuration(void);
void NVIC_Configuration(void);
volatile void Delay(vu32 nCount);
volatile uint32_t ticker=0;
void systickinit(void);
void tx_ledline(uint8_t * ledcmd);
void sevenseg_setbright(uint8_t val);
void sevenseg_disp(uint16_t val);
void my_itoa(int16_t value, uint8_t *destination);


/*disable all interrupts*/
static inline __attribute__((always_inline)) void __DISABLE_IRQ(void)
{__asm volatile("CPSID i");}

/*enable all interrupts*/
static inline __attribute__((always_inline)) void __ENABLE_IRQ(void)
{__asm volatile("CPSIE i");}

uint8_t disp_buff[32];
uint8_t eebuf[256];
static uint8_t i2c_status = I2C_OFF;
static struct eebuffer  * eep;


//------------------------------------------------------------------------------------------
volatile void Delay_ms(uint32_t ms)
{
	volatile uint32_t nCount;
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq (&RCC_Clocks);

        nCount=(RCC_Clocks.HCLK_Frequency/10000)*ms;
        for (; nCount!=0; nCount--);
}
//------------------------------------------------------------------------------------------
volatile void Delay(vu32 nCount)
{
  for(; nCount != 0; nCount--);
}
//------------------------------------------------------------------------------------------

void led_toggle(void)
{
    /* Read LED output (GPIOC PIN13) status */
    uint8_t led_bit = GPIO_ReadOutputDataBit(GPIOC, GPIO_Pin_13);
     
    /* If LED output set, clear it */
    if(led_bit == (uint8_t)Bit_SET)
    {
        GPIO_ResetBits(GPIOC, GPIO_Pin_13);
    }
    /* If LED output clear, set it */
    else
    {
        GPIO_SetBits(GPIOC, GPIO_Pin_13);
    }
}
void led_on(void)
{
        GPIO_ResetBits(GPIOC, GPIO_Pin_13);
}
void led_off(void)
{
        GPIO_SetBits(GPIOC, GPIO_Pin_13);
}


//------------------------------------------------------------------------------------------
void ioinit()
{
  /* Configure the system clocks */
  RCC_Configuration();
    
  /* NVIC Configuration */
  NVIC_Configuration();

  /* Enable peripheral clocks */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);

// DISABLE JTAG, Enable SWD
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
  GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE); // Disable JTAG/SWD so pins are available
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3|GPIO_Pin_2|GPIO_Pin_1|GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

// Configure PC.13 as Output push-pull
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

// Configure USART1+3 Rx (PB.7,11) for USART
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7|GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

// Configure USART1+3 Tx (PB.6,10) as alternate function push-pull
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6|GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

// Configure PB5, PB4, PB3 as digital Output for GM400
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5|GPIO_Pin_4|GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

// Configure PA10,PA9,PA8 as digital Input for GM400
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10|GPIO_Pin_9|GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_10MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

  GPIO_PinRemapConfig(GPIO_Remap_USART1, ENABLE);	// remap USART1 Pins to B6,B7

  Init_USART1();
  Init_USART3();

}
//------------------------------------------------------------------------------------------
void init_I2C1(void)
{



    // enable APB1 peripheral clock for I2C1, SCL and SDA pins
    I2C_DeInit(I2C1);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);

// already enabled
//    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB |  RCC_APB2Periph_AFIO,ENABLE);

    // Setup SCL und SDA pins
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_8 | GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_OD;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_PinRemapConfig(GPIO_Remap_I2C1,ENABLE);	// Pin 8+9 -> I2C

  // configure I2C1
  I2C_InitStructure.I2C_ClockSpeed = 100000;     // Hz
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_OwnAddress1 = 0x00;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Disable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;

  I2C_Cmd(I2C1, ENABLE);
  I2C_Init(I2C1, &I2C_InitStructure);
  I2C_AcknowledgeConfig(I2C_EE, ENABLE);
  i2c_status = I2C_OK;

  I2C_ClearFlag(I2C1, I2C_FLAG_AF);
  I2C_ClearFlag(I2C1, I2C_FLAG_ARLO);
  I2C_ClearFlag(I2C1, I2C_FLAG_BERR);

}

//------------------------------------------------------------------------------------------

void tx_ledline(uint8_t * ledcmd)
{
uint8_t i;
uint8_t cksum;

	cksum=0x7f;		// initialize checksum base

	USART1_PutChar('t');	// send 't'
	cksum ^='t';
	for(i=0; i<9; i++)	// first cmd, 8 characters following
	{
		USART1_PutChar(ledcmd[i]);
		cksum^=ledcmd[i];
	}
	USART1_PutChar('\r');	// 0x0d
	cksum^='\r';
	USART1_PutChar(cksum);	// checksum
}
	
void sevenseg_setbright(uint8_t val)
{
	uint8_t i;

	disp_buff[0]= 'c'+ 0x1;	// Address control 0. Display pair
	disp_buff[1]='%';		// set brightness
	disp_buff[2]=val;		// put value
	for(i=0;i <6;  i++)
		disp_buff[4+i]=0;	// fill up with "blanks"
	tx_ledline(disp_buff);
	disp_buff[0]++;		// next display
	tx_ledline(disp_buff);	// set 2nd display brightness

}

void sevenseg_disp(uint16_t val)	// 12.34  suppressing leading zeros
{
	uint8_t i,j;
	j=0;
	if(val&0x8000)
	{
		j=1;		// set neg flag
		val&=0x7fff;
	}

	disp_buff[0]='3';		// first display address
	disp_buff[1]=SegCode[(uint8_t)(val&0x000f)];		// Low
	disp_buff[2]=SegCode[(uint8_t)(val&0x00f0) >>4];		// High
	for(i=0;i <5;  i++)
		disp_buff[3+i]=0;	// fill up with "blanks"
	tx_ledline(disp_buff);
	disp_buff[0]++;		// 2nd display address
	disp_buff[1]=SegCode[(uint8_t)((val&0x0f00) >>8)]|0x80;	// Low
	disp_buff[2]=SegCode[(uint8_t)((val&0xf000) >>12)]; 	// High
#if 1
	if(disp_buff[2]==SegCode[0])
		disp_buff[2]=0;	// suppress leading blanks
#endif
	if(j)
		disp_buff[2]=SC_neg;	// set sign

	for(i=0;i <5;  i++)
		disp_buff[3+i]=0;	// fill up with "blanks"
	tx_ledline(disp_buff);
}
//------------------------------------------------------------------------------------------
uint32_t read_gm400()
{
	uint8_t clocks;
	uint32_t gmdata;

	gmdata=0;
       	GPIO_ResetBits(GPIOB, GPIO_Pin_5);	// clock to low
	Delay(5);
       	GPIO_SetBits(GPIOB, GPIO_Pin_5);	// clock to high
	Delay(5);
	for(clocks=0;clocks<25;clocks++)	// 12+13 Bit
	{
		gmdata= gmdata << 1;				// shift it
        	GPIO_ResetBits(GPIOB, GPIO_Pin_5);		// clock to low
		Delay(5);
		if(GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_10))	// read data
			gmdata|=0x1;				// or it in
       		GPIO_SetBits(GPIOB, GPIO_Pin_5);		// clock to high
		Delay(5);
	}
	return(gmdata);
}

//------------------------------------------------------------------------------------------
uint32_t gray_decode(uint32_t g)
{
    for (uint32_t bit = 1U << 31; bit > 1; bit >>= 1)
    {
        if (g & bit) g ^= bit >> 1;
    }
    return g;
}

//------------------------------------------------------------------------------------------
void read_eebuf()
{
	int16_t i;

	for(i=0;i<(256/EEP_PAGE_SZ);i++)
	{
		I2C_paketRead((uint16_t) i*EEP_PAGE_SZ, eebuf+i*EEP_PAGE_SZ, EEP_PAGE_SZ);
		Delay(1000);
	}
}
//------------------------------------------------------------------------------------------
void write_eebuf()
{
	int16_t i;

	for(i=0;i<(256/EEP_PAGE_SZ);i++)
	{
		I2C_paketWrite((uint16_t) i*EEP_PAGE_SZ , eebuf+i*EEP_PAGE_SZ, EEP_PAGE_SZ);
		Delay_ms(4);
	}
}
//------------------------------------------------------------------------------------------
void eeprom_init(void)
{
	uint8_t i;
	for(i=0;i<0xff;i++)		// erase buffer;
		eebuf[i]=0xff;
	
	eep->marker=(uint16_t)MAGIC;	// initialize buffer w. defaults

	for(i=2;i<14;i++)
		eep->res[i]=0xff;
	eep->wdgl[0]=12;
	eep->wdgl[1]=33;
	eep->wdgl[2]=33;
	eep->wdgl[3]=33;
	eep->wdgl[4]=33;
	eep->dlag[0]=385.1549623301;
	eep->dlag[1]=375.6056657434;
	eep->dlag[2]=366.0563691567;
	eep->dlag[3]=356.5070725700;
	eep->dlag[4]=346.9577759833;
	eep->sllg[0]=1210;
	eep->sllg[1]=1180;
	eep->sllg[2]=1150;
	eep->sllg[3]=1120;
	eep->sllg[4]=1090;
	eep->lges[0]=14520;
	eep->lges[1]=38940;
	eep->lges[2]=37950;
	eep->lges[3]=36960;
	eep->lges[4]=35970;
	eep->teuf[0]=0;
	eep->teuf[1]=14520;
	eep->teuf[2]=53460;
	eep->teuf[3]=91410;
	eep->teuf[4]=128370;
	write_eebuf();			// write buffer to eeprom

}
//------------------------------------------------------------------------------------------

void my_itoa(int16_t value, uint8_t *destination)
{
int16_t	dval;
	if (value <0)
	{
		dval=value*-1;
		destination[0] = '-';	//sign
	}
	else
	{
		dval=value;
		destination[0] = '0';	//sign
	}
	destination[1] = '0';	//houndrets
	destination[2] = '0';	//thens
	destination[3] = '0';	//ones
	destination[4] = '\0'; //\0
	while (dval >= 1000)
	{
		destination[0]++;
		dval -= 1000;
	}
	while (dval >= 100)
	{
		destination[1]++;
		dval -= 100;
	}
	while (dval >= 10)
	{
		destination[2]++;
		dval -= 10;
	}
	dval+= '0';
	destination[3] = dval;
}
//------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
int main(void)
{

int32_t simpulse;
int32_t slaenge;
uint32_t impulse;
uint16_t lindex;
uint32_t awlen;
double umdrehungen;

char obuf [256];
char * obp;
int16_t wlaen[5];

#ifdef DEBUG
  Debug();
#endif

	ioinit();
	systickinit();
	init_I2C1();

	wlaen[0]=1165;
	wlaen[1]=1095;
	wlaen[2]=1025;

	impulse=0;
	obp=obuf;

	USART3_PutString("\r\n\r\n*** Hello World! ***\r\n");
	strcpy(obp,"lolo!\r\n");
	USART3_PutString(obp);

	sevenseg_setbright(5);
	GPIO_SetBits(GPIOB,GPIO_Pin_4);		// V/~R
	Delay_ms(100);
	GPIO_SetBits(GPIOB,GPIO_Pin_3);		// Null
	Delay_ms(500);
	GPIO_ResetBits(GPIOB,GPIO_Pin_3);	// Null
	read_eebuf();
	eep=(void *)eebuf;
	if(eep->marker != MAGIC)
		eeprom_init();

	sevenseg_disp((uint16_t)eep->marker);

	Delay_ms(500);
	lcd_init_disp();
	lcd_bgl(0);
	lcd_set_cur(0,0);
	lcd_puts("ABC");
	lcd_set_cur(0,8);
	lcd_puts("!<>");
	lcd_set_cur(1,0);
	lcd_puts("123");
	lcd_set_cur(1,8);
	lcd_puts("abc");
	Delay_ms(500);
	lcd_clear();
	lcd_bgl(1);
	lcd_puts("Hallo Welt!");
	lcd_set_cur(1,0);
	lcd_put(0xe1);
	lcd_put(0xef);
	lcd_put(0xf5);
	lcd_put(0xe2);

#if 0
	I2C_paketRead((uint16_t)0x10,eebuf,8);
	Delay(1000);
	I2C_paketWrite((uint16_t)0x30,eebuf,8);
	Delay_ms(4);
	I2C_paketRead((uint16_t)0x110,eebuf,8);
	Delay(1000);
	I2C_paketWrite((uint16_t)0x130,eebuf,8);
	Delay_ms(4);
	I2C_paketRead((uint16_t)0x210,eebuf,8);
	Delay(1000);
	I2C_paketWrite((uint16_t)0x230,eebuf,8);
	Delay_ms(4);
	I2C_paketRead((uint16_t)0x310,eebuf,8);
	Delay(1000);
	I2C_paketWrite((uint16_t)0x330,eebuf,8);
	Delay_ms(4);
	k=I2C_EE_ByteRead(0x1f);
	Delay(1000);
#endif

	while (1)
	{
		if(ticker%3==0)
		{
			impulse=gray_decode(read_gm400());
			if(impulse>=16777215)
				simpulse=(int32_t)(impulse-33554432);
			else
				simpulse=(int32_t)impulse;

			umdrehungen=(double)simpulse/(UEBERSETZUNG);
			if((fabs(umdrehungen) <= (double) 14.0))
			{
				lindex=0;
				awlen=0;
				slaenge=(int32_t)((double)wlaen[lindex]*umdrehungen);
				
			}
			else
			{ 
				if((fabs(umdrehungen) <= (double) 31.0))
				{
					lindex=1;
					awlen=wlaen[0]*14;	// add laenge Lage 0
					if (umdrehungen < 0)
						slaenge=(int32_t)(wlaen[lindex]*(umdrehungen+14.0))-awlen;
					else
						slaenge=(int32_t)(wlaen[lindex]*(umdrehungen-14.0))+awlen;
				
				}
				else
				{
					lindex=2;
								// add laenge Lage 0+1
					awlen=wlaen[0]*14+wlaen[1]*31;
					if (umdrehungen < 0)
						slaenge=(int32_t)(wlaen[lindex]*(umdrehungen+45.0))-awlen;
					else
						slaenge=(int32_t)(wlaen[lindex]*(umdrehungen-45.0))+awlen;
				}
			}
	
	
			sprintf(obuf,"Simp:%ld; Umdr:%3.2f, Awlen: %lu Laenge:%ld\r\n",
				simpulse,umdrehungen,awlen,slaenge);
			USART3_PutString(obp);
		
			my_itoa((int16_t)(slaenge/10)&0xffff,disp_buff);
			if(slaenge<0)	// negative
			{
				sevenseg_disp(((uint16_t)(disp_buff[0]-0x30)<<12)| 
					((uint16_t)(disp_buff[1]-0x30)<<8)|
					((uint16_t)(disp_buff[2]-0x30)<<4)|
					((uint16_t)(disp_buff[3]-0x30))|0x8000);
			}
			else
			{
				sevenseg_disp(((uint16_t)(disp_buff[0]-0x30)<<12)| 
					((uint16_t)(disp_buff[1]-0x30)<<8)|
					((uint16_t)(disp_buff[2]-0x30)<<4)|
					((uint16_t)(disp_buff[3]-0x30)));
			}
		
    			GPIOA->BSRR=(0x0f<<16); // clear PA0-3
    			GPIOA->BSRR=(impulse&0x0f);
		}
  	}
}
//------------------------------------------------------------------------------------------
void SysTick_Handler(void) // 100ms
{
	led_toggle();
	ticker++;
}
//------------------------------------------------------------------------------------------
void RCC_Configuration(void)
{
  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);
 	
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
  }
}
//------------------------------------------------------------------------------------------
void NVIC_Configuration(void)
{
//#ifdef  VECT_TAB_RAM  
#if 0
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

void systickinit(void)
{

	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
	/*for Real Hardware, 1ms */
    	int onems = ((SysTick->CALIB) & 0x00ffffff);
    	//int reload = 999;  // Supposed to be once a second (999+1 ms)
    	int reload = 99;  // Supposed to be once a second (999+1 ms)

    	/* If the calibration value is non 0 then we can use it as a more accurate value for 1ms */
	if(onems != 0)
	{
		//reload = ((onems+1) * 1000)-1;
		reload = ((onems+1) * 100)-1;
	}
	SysTick->LOAD = reload;
	SysTick->CTRL =  SysTick_CTRL_ENABLE_Msk |  SysTick_CTRL_TICKINT_Msk; /* Start, with interrupts enabled */


}

